﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.IO;
using System.Data;

namespace XMLSerializationExs
{
    public partial class frmXMLSerialization : Form
    {
        public frmXMLSerialization()
        {
            InitializeComponent();
        }


        #region Basit XML Serileştirme
        private void btnBasicSerialization_Click(object sender, EventArgs e)
        {
            XMLSerializeMyObject();
            XMLDeSerializeMyObject();
        }


        // Basit bir serileştirilecek sınıf.
        [Serializable()]
        public class MyClass
        {
            public string strPublic;
            public MyClass()
            {
                strPublic = "Public Veri";
            }
        }

        private void XMLSerializeMyObject()
        {
            // Serileştirilecek nesne oluşturuluyor.
            MyClass ob = new MyClass();

            // Serileştirilmiş nesneyi taşıyacak stream oluşturuluyor.
            Stream stream = new FileStream("BasicSerialization.xml", FileMode.Create, FileAccess.Write, FileShare.Write);

            // Nesne serileştirilecek kaydedilecek stream'e yazılıyor.
            XmlSerializer xmlserializer = new XmlSerializer(typeof(MyClass));
            xmlserializer.Serialize(stream, ob);

            // Stream kapatılıyor.
            stream.Close();
        }

        private void XMLDeSerializeMyObject()
        {
            // Read the file back into a stream
            Stream stream = new FileStream("BasicSerialization.xml", FileMode.Open, FileAccess.Read, FileShare.Read);

            // Serileştirilmiş nesneyi ters-serileştirme için XmlSerializer oluşturuluyor.
            XmlSerializer xmlserializer = new XmlSerializer(typeof(MyClass));

            // Nesne ters-serileştirilip okunuyor.
            MyClass ob = (MyClass)xmlserializer.Deserialize(stream);

            // Stream kapatılıyor.
            stream.Close();
        }

        #endregion

        #region DataSet Serileştirme
        private void btnDataSetSerialization_Click(object sender, EventArgs e)
        {
            // DataSet oluşturuluyor.
            DataSet ds = new DataSet("MyDataSet");
            DataTable dt = new DataTable("MyDataTable");
            DataColumn dc1 = new DataColumn("ID", typeof(int));
            DataColumn dc2 = new DataColumn("Name", typeof(string));

            dt.Columns.Add(dc1);
            dt.Columns.Add(dc2);
            ds.Tables.Add(dt);

            // Bazı örnek kayıtlar ekleniyor.
            for (int i = 0; i < 3; i++)
            {
                DataRow row = dt.NewRow();
                row[0] = i;
                row[1] = i.ToString();
                dt.Rows.Add(row);
            }

            // DataSet Serileştiriliyor.
            XmlSerializer serializer = new XmlSerializer(typeof(DataSet));
            Stream stream = new FileStream("DataSetSerialization.xml", FileMode.CreateNew);
            serializer.Serialize(stream, ds);

            // Stream kapatılıyor.
            stream.Close();
        }

        #endregion

        #region Karmaşık Nesne Serileştirme

        [Serializable]
        public class Item
        {
            public string strProductName;
            public int nProductID;

            public Item()
            {
                strProductName = "";
                nProductID = 0;
            }
            public Item(string s, int n)
            {
                strProductName = s;
                nProductID = n;
            }
        }

        [Serializable]
        public class Order
        {

            // Serileştirilecek nesne karmaşık bir nesneye referans için referans edilen 
            // nesne serileştirilirken element olarak kullanılacağı belirtilmelidir.
            // Bu konu ile ilgili daha detaylı olarak aşağıdaki konu başlığında bulabilirsiniz;
            // XML Serileştirmeyi Etiket Kullanarak Kontrol Etmek

            [XmlElement(typeof(Item))]
            public List<Item> alOrders;

            public Order()
            {
                alOrders = new List<Item>();
            }

            [XmlInclude(typeof(Item))]
            public void Add(Item i)
            {
                alOrders.Add(i);
            }
        }

        private void btnComplexSerialization_Click(object sender, EventArgs e)
        {
            Stream stream = null;
            try
            {
                // Serileştirilecek nesne ve referans ettiği diğer nesne oluşturuluyor.
                Order order = new Order();
                order.Add(new Item("Elma", 10));
                order.Add(new Item("Portakal", 20));
                order.Add(new Item("Çilek", 30));

                // Nesne serileştiriliyor.
                XmlSerializer serializer = new XmlSerializer(typeof(Order));
                stream = new FileStream("ComplexSerialization.xml", FileMode.Create);
                serializer.Serialize(stream, order);
                stream.Close();
            }
            catch (Exception ex)
            {
                stream.Close();
            }
        }
        #endregion

        #region XML Namespace'lerle çalışmak
        private void btnWorkingWithXmlNamespaces_Click(object sender, EventArgs e)
        {
            // Kitap tanımlayalım.
            Book book1 = new Book();
            Price price1 = new Price();
            price1.currency = "TL";
            price1.price = 49.99M;
            book1.Title = "İleri C#";
            book1.price = price1;

            // Başka bir kitap daha tanımlayalım.
            Book book2 = new Book();
            Price price2 = new Price();
            price2.currency = "USD";
            price2.price = 39.99M;
            book2.Title = "MS SQL Server 2012";
            book2.price = price2;

            // Kitap listesine kitapları ekleyelim.
            Books books = new Books();
            books.alBooks.Add(book1);
            books.alBooks.Add(book2);

            /* XML namespace ve prefix tanımlayalım. */
            XmlSerializerNamespaces namespaces = new XmlSerializerNamespaces();
            namespaces.Add("KC", "www.kazimcesur.com");

            /* XML dosyaya serileştirelim. */
            Stream stream = new FileStream("Namespaces.xml", FileMode.Create, FileAccess.Write, FileShare.Write);
            XmlSerializer serializer = new XmlSerializer(typeof(Books));
            serializer.Serialize(stream, books, namespaces);
            stream.Close();
        }

        public class Price
        {
            // currency etiket ile verilen namespace'le serileştirilsin.
            [XmlAttribute(Namespace = "www.kazimcesur.com")]
            public string currency;

            // price etiket ile verilen namespace'le serileştirilsin.
            [XmlAttribute(Namespace = "www.kazimcesur.com")]
            public decimal price;
        }

        public class Book
        {
            // Title etiket ile verilen namespace'le serileştirilsin.
            [XmlElement(Namespace = "www.kazimcesur.com")]
            public string Title;

            // price etiket ile verilen namespace'le serileştirilsin.
            [XmlElement(Namespace = "www.kazimcesur.com")]
            public Price price;
        }

        public class Books
        {
            // alBooks etiket ile verilen namespace'le serileştirilsin.
            [XmlElement(typeof(Book), Namespace = "www.kazimcesur.com")]
            public List<Book> alBooks = new List<Book>();
        }
        #endregion


    }
}
